/* This file is Copyright 2001 Level Control Systems.  See the included LICENSE.TXT file for details. */
package com.lcs.muscle.iogateway;

import java.io.DataInput;
import java.io.DataOutput;
import java.io.IOException;
import com.lcs.muscle.support.UnflattenFormatException;
import com.lcs.muscle.message.Message;

/** A gateway that converts to and from the 'standard' MUSCLE flattened message byte stream. */
public class MessageIOGateway implements AbstractMessageIOGateway {
    /** Reads from the input stream until a Message can be assembled and returned.
      * @param in The input stream to read from.
      * @throws IOException if there is an error reading from the stream.
      * @throws UnflattenFormatException if there is an error parsing the data in the stream.
      * @return The next assembled Message.
      */
    public Message unflattenMessage(DataInput in) throws IOException, UnflattenFormatException
    {
       int numBytes = in.readInt();
       if (in.readInt() != _encoding) throw new IOException();
       Message pmsg = new Message();
       pmsg.unflatten(in, numBytes);
       return pmsg;
    }
    
    /** Converts the given Message into bytes and sends it out the stream.
      * @param out the data stream to send the converted bytes to.
      * @param message the Message to convert.
      * @throws IOException if there is an error writing to the stream.
      */
    public void flattenMessage(DataOutput out, Message msg) throws IOException
    {
       out.writeInt(msg.flattenedSize());
       out.writeInt(_encoding);
       msg.flatten(out);
    }
    
    private final int _encoding =  1164862256;  // 'Enc0'  ... our default (plain-vanilla) encoding 
}
